@@ -67,6 +67,7 @@
 #include <cerrno>
 #include <cstdlib>
 #include <cstring>
+#include <limits.h>
 
 extern "C"
 {
@@ -223,7 +224,6 @@ namespace lsvpd
 	string SysFSTreeCollector::make_basic_device_scsi_ds(Component *fillMe,
 							     string type, int subtype)
 	{
-		string ds;
 		int i, j;
 		string prefix;
 		string subtypeDS;
@@ -250,7 +250,7 @@ namespace lsvpd
 
 		j = 0;
 		if ((device_scsi_types[i].key == -1)
-		    && (!cdrom_device_types[j].key != -1))
+		    && (cdrom_device_types[j].key != -1))
 			subtypeDS = cdrom_device_types[j].val;
 		else if (device_scsi_types[i].key == -1)
 			subtypeDS = "Unknown Device";
@@ -300,7 +300,7 @@ namespace lsvpd
 				       (cdtype != cdrom_device_types[i].key))
 					i++;
 
-				if (!cdrom_device_types[i].key != -1)
+				if (cdrom_device_types[i].key != -1)
 					subtypeDS = cdrom_device_types[i].val;
 				else
 					subtypeDS = SCSI_CDROM_DEFAULT;
@@ -333,7 +333,6 @@ namespace lsvpd
 	bool matches(string s1, string s2)
 	{
 		int beg = 0, end;
-		int z;
 
 		//strings have matched to end - base case
 		if (s1 == s2)
@@ -370,7 +369,7 @@ namespace lsvpd
 
 			if ( (int) string::npos == (end = s1.find("*", beg))) {
 				//No more stars - base case
-				if ((int) string::npos != (z = s2.find(s1.substr(beg, end), 0))) {
+				if (string::npos != s2.find(s1.substr(beg, end), 0)) {
 					return true;
 				}
 			}
@@ -516,50 +515,6 @@ namespace lsvpd
 		return value;
 	}
 
-	/**
-	 * Retrieves a specified field in a comma-delimited list
-	 * @arg: num: Number of field to grab, starting with 0
-	 */
-	string retrieveField(char * data, int num)
-	{
-		int commaCount = 0;
-		int beg, end, curPos;
-		string str = data;
-
-		end = beg = curPos = 0;
-
-		//coutd << "Data: " << data << ", Len = " << strlen(data) <<  endl;
-
-		while ((commaCount <= num) && (data[curPos] != '\0')) {
-			if (str[curPos] == ',') {
-				commaCount++;
-				if (commaCount <= num)
-					beg = curPos + 1;
-				else
-					end = curPos;
-			}
-			if (str[curPos + 1] == '\0') {
-				end = curPos;
-			}
-			curPos++;
-
-		}
-
-		/*
-		 * Exceptions:  [02]IBM seems to be messing up
-		 *  - Note: Be as specific as possible here
-		 */
-		if (beg >= end)
-			return "";
-
-		while ((str[beg] == 2) && (beg < (int) strlen(data)) && (beg < end))
-			beg++;
-
-		str = str.substr(beg, end - beg);
-
-		return str;
-	}
-
 	/**
 	 * Takes a full format specifier template string, counts number of
 	 *	page codes within it
@@ -628,28 +583,33 @@ namespace lsvpd
 	int retrievePageCode(const string page_spec_template,
 			     string& page_code, string& format)
 	{
-		int i = 0, beg, end = 0;
+		string::size_type beg, end;
 
-		/* Get page code */
-		while (page_spec_template[i-1] != '0'
-		       && page_spec_template[i] != 'x'
-		       && i < (int) page_spec_template.length())
-			i++;
-		beg = i + 1;
-		while (page_spec_template[i] != '=' && i < (int) page_spec_template.length())
-			i++;
-		end = i;
-		if ((beg >= (int) page_spec_template.length())
-		    || (end >= (int) page_spec_template.length()))
+		try {
+			beg = page_spec_template.find("0x");
+			if (beg == string::npos)
+				throw string("Missing 0x");
+
+			end = page_spec_template.find("=", beg);
+			if (end == string::npos)
+				throw string("Missing =");
+
+			page_code = page_spec_template.substr(beg+2, end-beg-2);
+			beg = end;
+
+			end = page_spec_template.find(';', beg);
+			if (end == string::npos)
+				end = page_spec_template.length();
+
+			format =  page_spec_template.substr(beg+1, end-beg-1);
+		} catch (string err) {
+			Logger().log(err.c_str( ), LOG_ERR );
+			format.clear();
+			page_code.clear();
 			return -1;
-		page_code = page_spec_template.substr(beg, end - beg);
+		}
 
-		/* Get format string */
-		beg = end + 1;
-		while (page_spec_template[i] != ';' && i < (int) page_spec_template.length())
-			i++;
-		end = i;
-		format = page_spec_template.substr(beg, end - beg);return 0;
+		return 0;
 	}
 
 	/**
@@ -700,7 +660,6 @@ namespace lsvpd
 		string fieldName;
 		int fieldSize;
 		string dataVal;  //Data as read from data stream for single field
-		string str;
 
 		if (pageCode == 0)
 			dataCurLoc = 32; // Skip ll_inquiry standard header for base inquiry
@@ -789,7 +748,6 @@ namespace lsvpd
 					      string *subtypeDS)
 	{
 		string ds;
-		string dataTmp = string(data);
 		string z0;
 		Logger log;
 		string tmpStr, lvd;
@@ -840,43 +798,49 @@ namespace lsvpd
 	/**
 	 * Close a device that may have been opened for reading
 	 */
-	void device_close(int device_fd, int major, int minor, int mode)
+	void device_close(int device_fd, string& dev_path)
 	{
-		char name[256];
 		struct stat statbuf;
-		if (-1 != sprintf(name, "/tmp/node-%d-%d-%d",
-				  mode, major, minor)) {
-			if (stat(name, &statbuf) == 0) {
-				unlink(name);
-			}
-		}
 
+		if (stat(dev_path.c_str(), &statbuf) == 0) {
+			if(device_fd >= 0)
+				close(device_fd);
+			unlink(dev_path.c_str());
+		}
 	}
 
 	// Open a temp file through which we can ioctl() to device for reading
-	int device_open(int major, int minor, int mode)
+	int device_open(int major, int minor, int mode, string& dev_path)
 	{
-		char name[256];
+		char name[PATH_MAX];
+		/* Var is used to store unique name */
+		char template_d[] = "/tmp/node-XXXXXX";
 		int device_fd = 0;
 		int ret;
 		struct stat statbuf;
 
-		if (sprintf(name, "/tmp/node-%d-%d-%d", mode, major, minor) < 0) {
-			return -ERROR_ACCESSING_DEVICE;
-		}
+		device_fd = mkstemp(template_d);
+		if (device_fd  == -1)
+			return -1;
+		close(device_fd);
 
-		if (stat(name, &statbuf) == 0) {
-			unlink(name);
-		}
+		if (stat(template_d, &statbuf) == 0)
+			unlink(template_d);
+
+		if (snprintf(name, PATH_MAX - 1, "%s-%d-%d-%d",
+			     template_d, mode, major, minor) < 0)
+			return -ERROR_ACCESSING_DEVICE;
 
 		ret = mknod(name, 0760 | mode, makedev(major, minor));
-		if (ret != 0){
+		if (ret != 0)
 			return -UNABLE_TO_MKNOD_FILE;
-		}
+
+		/* Store temp device filename. It will be used in device_close */
+		dev_path = string(name);
 
 		device_fd = open(name, 0);
 		if (device_fd < 0) {
-			device_close(device_fd, major, minor, mode);
+			device_close(device_fd, dev_path);
 			return -UNABLE_TO_OPEN_FILE;
 		}
 
@@ -1051,7 +1015,6 @@ namespace lsvpd
 		string line;
 		scsi_template *tmp;
 		int dev_count = 0;
-		string tmp_str;
 		ostringstream err;
 		ifstream fin(filename.c_str());
 
@@ -1112,15 +1075,12 @@ namespace lsvpd
 		int evpd;
 		int i, len = 0;
 		char buffer[MAXBUFSIZE];
-		string msg, pageCodeRefs;
 		struct sg_scsi_id sg_dat;
-		string rev, pro, ven;
 		int subtype, res, num;
 		const scsi_template *devTemplate = NULL; /* current dev's template */
 		string pageCode, pageFormat, pageTemp, subtypeDS;
 		int pageCodeInt;
 		int rc;
-		string str;
 		char vendor[32], model[32], firmware[32];
 
 		memset(vendor, '\0', 32);
@@ -1136,9 +1096,12 @@ namespace lsvpd
 				return -SCSI_FILL_TEMPLATE_LOADING;
 		}
 
-		res = ioctl(device_fd, SG_GET_SCSI_ID, &sg_dat);
-		if (res < 0) {
-			return -SGUTILS_IOCTL_FAILED;
+		/* Check for scsi devices */
+		if (fillMe->devBus.getValue() == "scsi") {
+			res = ioctl(device_fd, SG_GET_SCSI_ID, &sg_dat);
+			if (res < 0) {
+				return -SGUTILS_IOCTL_FAILED;
+			}
 		}
 
 		memset(buffer, '\0', MAXBUFSIZE);
@@ -1215,7 +1178,7 @@ namespace lsvpd
 			while ((device_scsi_types_short[i].key != -1)
 			       && (subtype != device_scsi_types_short[i].key))
 				i++;
-			if (!device_scsi_types_short[i].key != -1)
+			if (device_scsi_types_short[i].key != -1)
 				subtypeDS = device_scsi_types_short[i].val;
 			else
 				subtypeDS = "Unknown";
@@ -1319,8 +1282,7 @@ namespace lsvpd
 		string str;
 		int tmp, useGeneric = 0;
 		int beg, end;
-		string devClass, link, genericPath;
-		vector<DataItem*>::const_iterator j, stop;
+		string devClass, genericPath;
 
 		if (fillMe->idNode.getValue().empty())
 			return -CLASS_NODE_UNDEFINED;
@@ -1482,12 +1444,10 @@ out:
 	 */
 	void SysFSTreeCollector::fillSCSIComponent( Component* fillMe, bool limitSCSISize = false)
 	{
-		int rc;
-		string data;
 		int device_fd;
-		ostringstream err;
 		Logger logger;
 		string msg;
+		string dev_path;
 		/* Not a SCSI device */
 		if (fillMe->devBus.getValue() == "usb")
 			return;
@@ -1496,12 +1456,13 @@ out:
 		//		coutd << "Query SCSI dev at: " << fillMe->idNode.getValue() << endl;
 
 		/* Need major:minor codes to query device */
-		if (!(rc = get_mm_scsi(fillMe))) {
+		if (!get_mm_scsi(fillMe)) {
 
 			// Open Device for reading
 			device_fd = device_open(fillMe->devMajor,
 						fillMe->devMinor,
-						fillMe->devAccessMode);
+						fillMe->devAccessMode,
+						dev_path);
 			if (device_fd < 0) {
 				msg = string("vpdupdate: Failed opening device: ")
 					+ fillMe->idNode.getValue();
@@ -1511,10 +1472,7 @@ out:
 
 			collectVpd(fillMe, device_fd, limitSCSISize);
 
-			device_close(device_fd,
-				     fillMe->devMajor,
-				     fillMe->devMinor,
-				     fillMe->devAccessMode);
+			device_close(device_fd, dev_path);
 		}
 
 		fillIPRData( fillMe );
