@@ -44,6 +44,7 @@
 #include <sys/sysmacros.h>
 #include <dirent.h>
 #include <libgen.h>		// for basename()
+#include <stdint.h>
 #include <unistd.h>
 #include <cerrno>
 #include <cstdlib>
@@ -210,7 +211,6 @@ namespace lsvpd
 						      Component* fillMe)
 	{
 		string classLink;
-		Logger l;
 		string msg;
 
 		/* Here we need to do any of the class specific filling necessary.
@@ -222,6 +222,11 @@ namespace lsvpd
 			classLink = fillMe->getClassNode();
 			if (classLink.length() > 0)
 				fillNetClass( fillMe,  classLink);
+		} else if (devClass == "nvme")
+		{
+			classLink = fillMe->getClassNode();
+			if (classLink.length() > 0)
+				fillNvmeClass(fillMe);
 		}
 
 		return fillMe;
@@ -351,7 +356,7 @@ namespace lsvpd
 	{
 		vector <string> children;
 		vector <Component*>::iterator parent,tmp;
-		string devNode, devName;
+		string devName;
 
 		/* Remove duplicate devices */
 
@@ -442,29 +447,34 @@ namespace lsvpd
 			 */
 			string link = string(devNode + "/device");
 			if (FSWalk::fs_isLink(link)) {
-				char target[512];
-				char *buf;
-				string targetDevPath;
+				char linkTarget[PATH_MAX];
 				Component *targetDev;
-				size_t n;
 
-				buf = strdup(link.c_str());
-				n = readlink(buf, target, 512);
-				if (n > 0 && n < 512) {
-					target[n] = 0;
-					targetDevPath = HelperFunctions::getAbsolutePath(target, buf);
-					targetDev = findComponent(devs, targetDevPath);
-					if (targetDev != NULL) {
-						/* get the device name */
-						char *tmp = strdup(devNode.c_str());
-						char *name = basename(tmp);
+				if (realpath( link.c_str(), linkTarget ) == NULL) {
+					string msg = string("realpath operation")
+					 + string(" got failed on ") + link;
+					Logger().log(msg, LOG_ERR);
+					continue;
+				}
 
-						/* Add 'name' to the AIXName list for targetDev */
-						targetDev->addAIXName(name, 90);
-						free(tmp);
+				string targetDevPath = string(linkTarget);
+				targetDev = findComponent(devs, targetDevPath);
+				if (targetDev != NULL) {
+					/* get the device name */
+					char *tmp = strdup(devNode.c_str());
+					if(tmp == NULL) {
+						string msg = string("strdup")
+						+ string(" failed on ")
+						+ devNode;
+						Logger().log(msg, LOG_ERR);
+						continue;
 					}
+
+					/* Add 'name' to the AIXName list
+					 * for targetDev */
+					targetDev->addAIXName(basename(tmp), 90);
+					free(tmp);
 				}
-				free(buf);
 			}
 
 		}
@@ -575,17 +585,17 @@ namespace lsvpd
 
 		/* Get bus name if available */
 		if (HelperFunctions::file_exists(link)) {
-
 			if (FSWalk::fs_isLink(link)) {
-				char *buf, linkTarget[512];
+                                char linkTarget[PATH_MAX];
 
-				buf = strdup(link.c_str());
-				int len = readlink(buf, linkTarget, sizeof(
-									   linkTarget));
-				linkTarget[len] = '\0';
-				absTargetPath = HelperFunctions::getAbsolutePath(linkTarget,
-										 buf);
-				free(buf);
+				if (realpath( link.c_str(), linkTarget ) == NULL) {
+					string msg = string("realpath operation")
+						+ string(" got failed on ")
+						+ link;
+                                        Logger().log(msg, LOG_ERR);
+					goto esc_subsystem_info;
+                                }
+                                string absTargetPath = string(linkTarget);
 
 				/* 
 				 * Grab last 2 parts of link.
@@ -660,6 +670,8 @@ namespace lsvpd
 			}
 		}
 
+esc_subsystem_info:
+
 		if (fillMe->mDescription.getValue() == "Virtual") {
 			fillMe->mDescription.setValue("Virtual Device",
 						      2, __FILE__, __LINE__);
@@ -673,29 +685,31 @@ namespace lsvpd
 		/* Looking for device driver link */
 		link = string (fillMe->sysFsNode.getValue() + "/driver");
 		if (HelperFunctions::file_exists(link)) {
-
 			if (FSWalk::fs_isLink(link)) {
-				char *buf, linkTarget[512];
+				char linkTarget[PATH_MAX];
 				string driver;
 
-				buf = strdup(link.c_str());
-				int len = readlink(buf, linkTarget, sizeof(
-									   linkTarget));
-				linkTarget[len] = '\0';
-				absTargetPath = HelperFunctions::getAbsolutePath(linkTarget,
-										 buf);
-				free(buf);
-				/* Now grab last part of link */
-				lastSlash = absTargetPath.rfind("/", absTargetPath.length()) + 1;
-				driver = absTargetPath.substr(lastSlash,
-							      absTargetPath.length() - lastSlash);
-				fillMe->devDriver.setValue(driver, INIT_PREF_LEVEL,
-							   __FILE__, __LINE__);
+				if (realpath( link.c_str(), linkTarget ) != NULL) {
+					string absTargetPath = string(linkTarget);
 
-				if (driver == "hvc_console") {
-					fillMe->addAIXName(driver,90);
-					fillMe->mDescription.setValue("Hypervisor Virtual Console",
-								      90, __FILE__, __LINE__);
+					/* Now grab last part of link */
+					lastSlash = absTargetPath.rfind("/",
+						absTargetPath.length()) + 1;
+					driver = absTargetPath.substr(lastSlash,
+							absTargetPath.length() - lastSlash);
+					fillMe->devDriver.setValue(driver,
+								   INIT_PREF_LEVEL,
+								   __FILE__, __LINE__);
+
+					if (driver == "hvc_console") {
+						fillMe->addAIXName(driver,90);
+						fillMe->mDescription.setValue("Hypervisor Virtual Console",
+								90, __FILE__, __LINE__);
+					}
+				} else {
+					string msg = string ("realpath operation")
+						+ string(" got failed on ") + link;
+					Logger().log(msg, LOG_ERR);
 				}
 			}
 		}
@@ -707,12 +721,16 @@ namespace lsvpd
 			int len, start;
 			string name;
 
-			len = readlink(link.c_str(), linkTarget, sizeof(linkTarget));
-			linkTarget[len] = '\0';
+			len = readlink(link.c_str(), linkTarget,
+				       sizeof(linkTarget) - 1);
+			if (len > 0) {
+				linkTarget[len] = '\0';
 
-			name = string(linkTarget);
-			start = name.rfind("/", name.length()) + 1;
-			fillMe->addAIXName(name.substr(start, name.length() - start), 90);
+				name = linkTarget;
+				start = name.rfind("/", name.length()) + 1;
+				fillMe->addAIXName(name.substr(start,
+						name.length() - start), 90);
+			}
 		}
 
 
@@ -861,7 +879,7 @@ namespace lsvpd
 		string newDevDir;
 		vector<string> listing;
 		Component *tmpDev;
-		string tmp, devName, parentDev;
+		string devName, parentDev;
 		char *parent;
 
 		parent = strdup(parentDir.c_str());
@@ -914,7 +932,7 @@ namespace lsvpd
 	void SysFSTreeCollector::findDevicePaths(vector<Component*>& devs)
 	{
 		vector<string> fullList;
-		string curPath, tmpDirName, linkName, filePath, devPath;
+		string curPath, devPath;
 		FSWalk fsw = FSWalk();
 
 		/* Full list of the various categories of devices */
@@ -1016,11 +1034,14 @@ namespace lsvpd
 			{
 				char *f, *p;
 				f = strdup( fname.c_str( ) );
-				if ( f == NULL )
+				if (f == NULL) {
+					closedir(d);
 					return NULL;
+				}
 				p = strdup( sysDir.c_str( ) );
 				if ( p == NULL ) {
 					free (f);
+					closedir(d);
 					return NULL;
 				}
 				link = HelperFunctions::getAbsolutePath( f, p );
@@ -1044,7 +1065,6 @@ namespace lsvpd
 		Logger logger;
 		struct dirent* entry;
 		DIR* d;
-		string link;
 		bool filled = false;
 
 		d = opendir( sysDir.c_str( ) );
@@ -1258,6 +1278,115 @@ namespace lsvpd
 		}
 	}
 
+	unsigned int SysFSTreeCollector::parsePciVPDBuffer( Component* fillMe,
+						    char *buf, int size )
+	{
+		char key[ 3 ] = { '\0' };
+		/* Each VPD field will be at most 255 bytes long */
+		char val[ 256 ];
+		char *end;
+		unsigned char length;
+		uint16_t len;
+		string field;
+
+		/*
+		 * The format of the VPD Data is a series of sections, with
+		 * each section containing keyword followed by length as shown
+		 * below:
+		 *
+		 *   _ section start here
+		 *  |
+		 *  v
+		 *  ----------------------------------------------------------
+		 * | Section ID(1) | len(2) | data(len) | section1 tag(1) |   |
+		 *  ----------------------------------------------------------
+		 * | len (1) | key(2) | record_len(1) | data(record_len) |    |
+		 *  ----------------------------------------------------------
+		 * | key(2) | ....... | section2 tag(1) | len(1) | data(len)| |
+		 *  ----------------------------------------------------------
+		 * |.....| sectionN tag (1) | len(1) | data(len) | end tag(1) |
+		 *  ----------------------------------------------------------
+		 */
+
+		if (*buf != 0x82)
+			return 0;
+
+		/* Increment buffer to point to offset 1 to read Product
+		 * Name length
+		 */
+		buf++;
+
+		/* Data length is 2 bytes (byte 1 LSB, byte 2 MSB) */
+		len = *buf | (buf[1] << 8);
+
+		/* Increment buffer to point to read Product Name */
+		buf += 2;
+
+		/* Increment buffer to point to VPD R Tag */
+		buf += len;
+
+		/* Increment buffer to point to VPD R Tag data length */
+		buf++;
+
+		/* Increment buffer to point to first VPD keyword */
+		/* Increment by 2 because data length is of size 2 bytes */
+		buf += 2;
+
+		end = buf + size;
+
+		while( buf < end && *buf != 0x78 && *buf != 0x79 )
+		{
+			memset( key, '\0', 3 );
+			memset( val, '\0', 256 );
+
+			if( buf + 3 > end )
+			{
+				goto ERROR;
+			}
+
+			key[ 0 ] = buf[ 0 ];
+			key[ 1 ] = buf[ 1 ];
+			length = buf[ 2 ];
+			buf += 3;
+
+			if( buf + length > end )
+			{
+				goto ERROR;
+			}
+
+			memcpy( val, buf, length );
+			buf += length;
+
+			field = sanitizeVPDField(val, length);
+			setVPDField( fillMe, key, field, __FILE__, __LINE__ );
+		}
+
+		return size;
+ERROR:
+		Logger logger;
+		logger.log( "Attempting to parse corrupt VPD buffer.", LOG_WARNING );
+		return 0;
+	}
+
+	/* Parse VPD file */
+	void SysFSTreeCollector::fillPciDevVpd( Component* fillMe )
+	{
+		char *vpdData;
+		int size;
+		string path;
+
+		path = fillMe->sysFsNode.getValue() + "/vpd";
+		if (HelperFunctions::file_exists(path) != true)
+			return;
+
+		size = getBinaryData(path, &vpdData);
+		if (size == 0)
+			return;
+
+		parsePciVPDBuffer( fillMe, vpdData, size );
+		delete [] vpdData;
+	}
+
 	void SysFSTreeCollector::fillPCIDev( Component* fillMe,
 					     const string& sysDir )
 	{
@@ -1348,6 +1477,9 @@ namespace lsvpd
 
 		fillPCIDS( fillMe );
 
+		/* Fill PCI device VPD info */
+		fillPciDevVpd(fillMe);
+
 		// Read the pci config file for Device Specific (YC)
 		os.str( "" );
 		os << fillMe->sysFsNode.dataValue << "/config";
@@ -1424,6 +1556,9 @@ namespace lsvpd
 		else if( val == "tty" )
 			fillMe->mDescription.setValue( "Serial Device", 20,
 						       __FILE__, __LINE__ );
+		else if( val == "nvme" )
+			fillMe->mDescription.setValue( "NVMe Device", 20,
+						       __FILE__, __LINE__ );
 	}
 
 	void SysFSTreeCollector::fillUSBDev( Component* fillMe,
@@ -1479,6 +1614,83 @@ namespace lsvpd
 #ifndef SIOCETHTOOL
 #define SIOCETHTOOL     0x8946
 #endif
+
+	/**
+	 * Fills NVMe device info
+	 */
+	void SysFSTreeCollector::fillNvmeClass( Component* fillMe )
+	{
+		FSWalk fsw = FSWalk();
+		vector<string> listing;
+		string dev_syspath;
+		string dev_childname;
+		size_t start;
+		int beg, end;
+		string str;
+		string newDevDir;
+		bool dev_found = false;
+		int device_fd;
+		string dev_path;
+		string msg;
+
+		dev_syspath = fillMe->sysFsNode.getValue();
+		fsw.fs_getDirContents(dev_syspath, 'd', listing);
+		if (listing.size() <= 0) {
+			Logger().log("fillNvmeClass: NVMe dev not found.",
+				     LOG_WARNING);
+			return;
+		}
+
+		while (listing.size() > 0) {
+			dev_childname = listing.back();
+			listing.pop_back();
+			start = dev_childname.find("nvme");
+			if (start != string::npos) {
+				dev_found = true;
+				break;
+			}
+		}
+
+		if (dev_found != true) {
+			Logger().log("fillNvmeClass: NVMe dev matching failed.",
+				     LOG_WARNING);
+			return;
+		}
+
+		/**
+		 * Get major/minor number
+		 */
+		newDevDir = dev_syspath + "/" + dev_childname;
+		str = getAttrValue( newDevDir, "dev" );
+		beg = end = 0;
+		while (end < (int) str.length() && str[end] != ':')
+			end++;
+
+		fillMe->devMajor = atoi(str.substr(beg, end).c_str());
+
+		beg = end + 1;
+		end = beg;
+		while (end < (int) str.length() && str[end] != ':') {
+			end++;
+		}
+
+		fillMe->devMinor = atoi(str.substr(beg, end).c_str());
+		fillMe->devAccessMode = S_IFBLK;
+
+		device_fd = device_open(fillMe->devMajor, fillMe->devMinor,
+					fillMe->devAccessMode, dev_path);
+		if (device_fd < 0) {
+			msg = string("vpdupdate: Failed opening device: ")
+				+ fillMe->idNode.getValue();
+			Logger().log( msg, LOG_WARNING );
+			return;
+		}
+
+		collectVpd(fillMe, device_fd, false);
+		device_close(device_fd, dev_path);
+		return;
+	}
+
 	void SysFSTreeCollector::fillNetClass( Component* fillMe,
 					       const string& classDir )
 	{
@@ -1570,9 +1782,6 @@ namespace lsvpd
 
 	void SysFSTreeCollector::fillFirmware( Component* fillMe )
 	{
-		Logger l;
-		string msg;
-
 		string classNode = fillMe->getClassNode();
 		if (classNode.length() > 0) {
 			fillMe->mFirmwareVersion.setValue( getAttrValue( classNode,
